"use strict";

var _react = _interopRequireDefault(require("react"));

var _ReactPolling = _interopRequireDefault(require("./ReactPolling"));

var _enzyme = require("enzyme");

var mockData = _interopRequireWildcard(require("../__mock_data__/data"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

describe('<ReactPolling />', function () {
  var onSuccess = jest.fn(),
      onFailure = jest.fn,
      url = 'http://localhost/session/status';
  describe('success test cases', function () {
    beforeAll(function () {
      window.fetch = jest.fn().mockImplementation(function () {
        return Promise.resolve({
          json: function json() {
            return new Promise(function (resolve) {
              resolve(mockData.continuePollingResponse);
            });
          },
          ok: true
        });
      });
    });
    test('should render properly with render props', function () {
      var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
        url: url,
        onSuccess: onSuccess,
        onFailure: onFailure,
        render: function render(_ref) {
          var isPolling = _ref.isPolling;
          return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
            id: "isPolling"
          }, " I am polling") : _react.default.createElement("div", {
            id: "isNotPolling"
          }, " I am not polling "));
        }
      }));
      expect(wrapper.find('p').length).toBe(1);
      expect(wrapper.find('#isPolling').length).toBe(1);
    });
    test('should render properly with children props', function () {
      var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
        url: url,
        onSuccess: onSuccess,
        onFailure: onFailure
      }, function (_ref2) {
        var isPolling = _ref2.isPolling;
        return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
          id: "isPolling"
        }, " I am polling") : _react.default.createElement("div", {
          id: "isNotPolling"
        }, " I am not polling "));
      }));
      expect(wrapper.find('p').length).toBe(1);
      expect(wrapper.find('#isPolling').length).toBe(1);
    });
    describe('initial tests for checking initial variables before we start polling', function () {
      var wrapper, mockedComponentDidMount;
      beforeEach(function () {
        mockedComponentDidMount = jest.spyOn(_ReactPolling.default.prototype, 'componentDidMount');
        mockedComponentDidMount.mockImplementation();
        wrapper = (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
          url: url,
          onSuccess: onSuccess,
          onFailure: onFailure,
          render: function render(_ref3) {
            var isPolling = _ref3.isPolling;
            return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
              id: "isPolling"
            }, " I am polling") : _react.default.createElement("div", {
              id: "isNotPolling"
            }, " I am not polling "));
          }
        }));
      });
      afterEach(function () {
        wrapper = null;
        mockedComponentDidMount.mockRestore();
      });
      test('should have right initial state', function () {
        var expectedState = {
          isPolling: false
        };
        expect(wrapper.state()).toEqual(expectedState);
      });
      test('should have right initial config', function () {
        var expectedConfig = {
          url: url,
          interval: 3000,
          shouldRetry: false,
          retryCount: 0,
          onSuccess: onSuccess,
          onFailure: onFailure,
          api: {
            method: 'GET'
          }
        };
        expect(wrapper.instance().config).toMatchObject(expectedConfig);
      });
      test('should have all the function defined', function () {
        expect(wrapper.instance().initConfig).toBeDefined();
        expect(wrapper.instance().config).toBeDefined();
        expect(wrapper.instance().startPolling).toBeDefined();
        expect(wrapper.instance().stopPolling).toBeDefined();
        expect(wrapper.instance().runPolling).toBeDefined();
      });
    });
    describe('tests with polling started', function () {
      test('startPolling function to have been called', function () {
        var mockedStartPolling = jest.spyOn(_ReactPolling.default.prototype, 'startPolling');
        mockedStartPolling.mockImplementation();
        (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
          url: url,
          onSuccess: onSuccess,
          onFailure: onFailure,
          render: function render(_ref4) {
            var isPolling = _ref4.isPolling;
            return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
              id: "isPolling"
            }, " I am polling") : _react.default.createElement("div", {
              id: "isNotPolling"
            }, " I am not polling "));
          }
        }));
        expect(mockedStartPolling).toHaveBeenCalledTimes(1);
        mockedStartPolling.mockRestore();
      });
      test('startPolling function to throw an error when no url provided', function () {
        expect(function () {
          (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
            url: null,
            onSuccess: onSuccess,
            onFailure: onFailure,
            render: function render(_ref5) {
              var isPolling = _ref5.isPolling;
              return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
                id: "isPolling"
              }, " I am polling") : _react.default.createElement("div", {
                id: "isNotPolling"
              }, " I am not polling "));
            }
          }));
        }).toThrowError();
      });
      test('startPolling should set isPolling state to true, _ismounted to true and call runPolling function', function () {
        var mockedRunPolling = jest.spyOn(_ReactPolling.default.prototype, 'runPolling');
        mockedRunPolling.mockImplementation();
        var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
          url: url,
          onSuccess: onSuccess,
          onFailure: onFailure,
          render: function render(_ref6) {
            var isPolling = _ref6.isPolling;
            return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
              id: "isPolling"
            }, " I am polling") : _react.default.createElement("div", {
              id: "isNotPolling"
            }, " I am not polling "));
          }
        }));
        wrapper.update();
        expect(wrapper.state().isPolling).toBeTruthy();
        expect(wrapper.instance()._ismounted).toBeTruthy();
        expect(mockedRunPolling).toHaveBeenCalledTimes(1);
        mockedRunPolling.mockRestore();
        mockedRunPolling.mockClear();
      });
      test('stopPolling should set isPolling to false, _ismounted to false and poll to null', function () {
        var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
          url: url,
          onSuccess: onSuccess,
          onFailure: onFailure,
          render: function render(_ref7) {
            var isPolling = _ref7.isPolling;
            return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
              id: "isPolling"
            }, " I am polling") : _react.default.createElement("div", {
              id: "isNotPolling"
            }, " I am not polling "));
          }
        }));
        wrapper.instance().stopPolling();
        wrapper.update();
        expect(wrapper.state().isPolling).toBeFalsy();
        expect(wrapper.instance()._ismounted).toBeTruthy();
        expect(wrapper.instance().poll).toBeNull();
      });
    });
    describe('timer events', function () {
      beforeEach(function () {
        jest.useFakeTimers();
      });
      afterEach(function () {
        jest.clearAllTimers();
      });
      test('run Polling should call setTimeout and make api calls at every interval',
      /*#__PURE__*/
      _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee() {
        var mockedRunPolling, mockedOnSuccess;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                mockedRunPolling = jest.spyOn(_ReactPolling.default.prototype, 'runPolling');
                mockedOnSuccess = jest.fn(function () {
                  return true;
                });
                (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
                  url: url,
                  onSuccess: mockedOnSuccess,
                  onFailure: onFailure,
                  render: function render(_ref9) {
                    var isPolling = _ref9.isPolling;
                    return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
                      id: "isPolling"
                    }, " I am polling") : _react.default.createElement("div", {
                      id: "isNotPolling"
                    }, " I am not polling "));
                  }
                }));
                expect(mockedRunPolling).toHaveBeenCalledTimes(1);
                expect(setTimeout).toHaveBeenCalledTimes(1);
                jest.runAllTimers();
                expect(fetch).toHaveBeenCalled();
                _context.next = 9;
                return Promise.resolve();

              case 9:
                _context.next = 11;
                return Promise.resolve();

              case 11:
                _context.next = 13;
                return Promise.resolve();

              case 13:
                _context.next = 15;
                return Promise.resolve();

              case 15:
                expect(mockedOnSuccess).toHaveBeenCalled();
                _context.next = 18;
                return Promise.resolve();

              case 18:
                expect(mockedRunPolling).toHaveBeenCalledTimes(2);
                expect(setTimeout).toHaveBeenCalledTimes(2);
                jest.runAllTimers();
                expect(fetch).toHaveBeenCalled();
                _context.next = 24;
                return Promise.resolve();

              case 24:
                _context.next = 26;
                return Promise.resolve();

              case 26:
                _context.next = 28;
                return Promise.resolve();

              case 28:
                _context.next = 30;
                return Promise.resolve();

              case 30:
                expect(mockedOnSuccess).toHaveBeenCalled();
                mockedRunPolling.mockRestore();
                mockedRunPolling.mockClear();

              case 33:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      })));
    });
  });
  describe('error test cases', function () {
    beforeAll(function () {
      window.fetch = jest.fn().mockImplementation(function () {
        return new Promise(function (resolve, reject) {
          reject(true);
        });
      });
    });
    beforeEach(function () {
      jest.useFakeTimers();
    });
    afterEach(function () {
      jest.clearAllTimers();
    });
    test('onFailure should retry equal to config.retryTimes',
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2() {
      var mockedRunPolling, mockedStopPolling, retryCount, wrapper, i, _i, _i2;

      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              mockedRunPolling = jest.spyOn(_ReactPolling.default.prototype, 'runPolling');
              mockedStopPolling = jest.spyOn(_ReactPolling.default.prototype, 'stopPolling');
              retryCount = 4;
              wrapper = (0, _enzyme.shallow)(_react.default.createElement(_ReactPolling.default, {
                url: url,
                onSuccess: onSuccess,
                onFailure: onFailure,
                retryCount: retryCount,
                render: function render(_ref11) {
                  var isPolling = _ref11.isPolling;
                  return _react.default.createElement("div", null, _react.default.createElement("p", null, "Polling Component"), isPolling ? _react.default.createElement("div", {
                    id: "isPolling"
                  }, " I am polling") : _react.default.createElement("div", {
                    id: "isNotPolling"
                  }, " I am not polling "));
                }
              }));
              expect(mockedRunPolling).toHaveBeenCalledTimes(1);
              expect(setTimeout).toHaveBeenCalledTimes(1);
              i = 0;

            case 7:
              if (!(i < retryCount)) {
                _context2.next = 23;
                break;
              }

              jest.runAllTimers();
              expect(fetch).toHaveBeenCalledTimes(i + 1);
              _i = 0;

            case 11:
              if (!(_i < retryCount + 1)) {
                _context2.next = 17;
                break;
              }

              _context2.next = 14;
              return Promise.resolve(true);

            case 14:
              _i++;
              _context2.next = 11;
              break;

            case 17:
              expect(mockedRunPolling).toHaveBeenCalledTimes(i + 2);
              expect(mockedStopPolling).toHaveBeenCalledTimes(0);
              expect(wrapper.state().isPolling).toBeTruthy();

            case 20:
              i++;
              _context2.next = 7;
              break;

            case 23:
              jest.runAllTimers();
              _i2 = 0;

            case 25:
              if (!(_i2 < retryCount + 1)) {
                _context2.next = 31;
                break;
              }

              _context2.next = 28;
              return Promise.resolve(true);

            case 28:
              _i2++;
              _context2.next = 25;
              break;

            case 31:
              expect(mockedStopPolling).toHaveBeenCalledTimes(1);
              expect(wrapper.instance().config._ismounted).toBeFalsy();
              expect(wrapper.state().isPolling).toBeFalsy();
              mockedRunPolling.mockRestore();
              mockedRunPolling.mockClear();

            case 36:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    })));
  });
});